/*  einsc -- a compiler for eins
 *    Source code position management
 *
 *    Matthias Zenger, Michel Schinz, Philippe Altherr
 *    $Id: Position.java,v 1.2 2005/04/27 13:56:08 dragos Exp $
 */

package einsc;


/** This class provides methods for encoding a location in
 *  a source file (line/column) into a single integer.
 */
public final class Position {

    /** number of bits reserved for the column.
     */
    private static final int columnBits = 12;

    private static final int columnMask = (1 << columnBits) - 1;

    /** undefined position
     */
    public static final int UNDEFINED = 0;

    /** first position in a source file
     */
    public static final int FIRST = (1 << columnBits) | 1;

    /** encode a line and column number into a single int
     */
    public static int encode(int line, int col) {
        return (line << columnBits) | (col & columnMask);
    }

    /** get the line number out of an encoded position
     */
    public static int line(int pos) {
        return pos >>> columnBits;
    }

    /** return the column number of an encoded position
     */
    public static int column(int pos) {
        return pos & columnMask;
    }
}
