/*
 * Created on Feb 28, 2005
 *
 * $Id: Traverser.java,v 1.2 2005/04/27 13:56:08 dragos Exp $
 */
package einsc;

import einsc.Tree.*;


/**
 * @author dragos
 *
 * Traverses the whole tree. It is different than the @see DefaultVisitor
 * because, by default, it traverses the subtrees of the current tree.
 */
public class Traverser implements Visitor {

    public void traverse(Tree tree) {
        tree.apply(this);
    }

    private void traverse(Tree[] trees) {
        for (int i = 0; i < trees.length; ++i)
            traverse(trees[i]);
    }

	public void caseBad(Bad tree) {
	}

	public void caseProgram(Program tree) {
		traverse(tree.decls);
        traverse(tree.main);
	}

	public void caseClassDef(ClassDef tree) {
        traverse(tree.fields);
	}

    public void caseFunDef(FunDef tree) {
        // we don't visit the parameters, only the body
        traverse(tree.body);
	}

	public void caseFormal(Formal tree) {
		traverse(tree.vartype);
	}

	public void caseIntType(IntType tree) {
	}

	public void caseTypeIdent(TypeIdent tree) {
	}

	public void caseFunType(FunType tree) {
		traverse(tree.paramtypes);
        traverse(tree.restype);
	}
    
    public void caseArrayType(ArrayType tree) {
        traverse(tree.baseType);
    }

	public void caseWhile(While tree) {
		traverse(tree.cond);
        traverse(tree.stats);
	}

	public void caseVarDef(VarDef tree) {
        // don't visit the declaration, only the initializer
        traverse(tree.init);
	}

	public void caseAssign(Assign tree) {
		traverse(tree.expr);
	}

	public void caseDo(Do tree) {
		traverse(tree.expr);
	}

	public void caseIf(If tree) {
		traverse(tree.cond);
        traverse(tree.thenp);
        traverse(tree.elsep);
    }

    public void caseUnop(Unop tree) {
    	traverse(tree.expr);
	}

	public void caseBinop(Binop tree) {
		traverse(tree.left);
        traverse(tree.right);
    }

	public void caseIdent(Ident tree) {
	}

	public void caseNullLit(NullLit tree) {
	}

	public void caseIntLit(IntLit tree) {
	}

	public void caseApply(Apply tree) {
		traverse(tree.fun);
        traverse(tree.args);
    }

	public void caseBlock(Block tree) {
		traverse(tree.stats);
        traverse(tree.main);
	}

	public void caseNew(New tree) {
		traverse(tree.args);
	}

    public void caseSelect(Select tree) {
    	traverse(tree.prefix);
    }

    public void caseIndex(Index tree) {
        traverse(tree.array);
        traverse(tree.indexes);
    }

    public void caseNewArray(NewArray tree) {
        traverse(tree.baseType);
        traverse(tree.args);
    }
}
